#ifndef SW_channelSAN_H_
#define SW_channelSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern double failure_coverage;
extern double SS_repair_rate;
extern double USF_repair_rate;
extern double channel_failure_rate;
extern double CC_failure_rate;
extern UserDistributions* TheDistribution;

void MemoryError();


/*********************************************************************
               SW_channelSAN Submodel Definition                   
*********************************************************************/

class SW_channelSAN:public SANModel{
public:

class CCF_propagateActivity:public Activity {
public:

  Place* CCF_active;
  short* CCF_active_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* channel_failure;
  short* channel_failure_Mobius_Mark;

  double* TheDistributionParameters;
  CCF_propagateActivity();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
}; // CCF_propagateActivityActivity

class OK_failureActivity:public Activity {
public:

  Place* Channel_OK;
  short* Channel_OK_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* channel_failure;
  short* channel_failure_Mobius_Mark;

  double* TheDistributionParameters;
  OK_failureActivity();
  ~OK_failureActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // OK_failureActivityActivity

class Safe_fail_OKActivity:public Activity {
public:

  Place* Safe_state;
  short* Safe_state_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* Channel_OK;
  short* Channel_OK_Mobius_Mark;

  double* TheDistributionParameters;
  Safe_fail_OKActivity();
  ~Safe_fail_OKActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // Safe_fail_OKActivityActivity

class DiagnosisActivity_case1:public Activity {
public:

  Place* channel_failure;
  short* channel_failure_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* Configuration_state;
  short* Configuration_state_Mobius_Mark;
  Place* Safe_state;
  short* Safe_state_Mobius_Mark;
  Place* Unsafe_state_2;
  short* Unsafe_state_2_Mobius_Mark;
  Place* Channel_OK;
  short* Channel_OK_Mobius_Mark;

  double* TheDistributionParameters;
  DiagnosisActivity_case1();
  ~DiagnosisActivity_case1();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // DiagnosisActivity_case1Activity

class DiagnosisActivity_case2:public Activity {
public:

  Place* channel_failure;
  short* channel_failure_Mobius_Mark;
  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* Unsafe_state;
  short* Unsafe_state_Mobius_Mark;

  double* TheDistributionParameters;
  DiagnosisActivity_case2();
  ~DiagnosisActivity_case2();
  double Rate(){return 0;}
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double DeterministicParamValue();
}; // DiagnosisActivity_case2Activity

class Repair_unsafeActivity:public Activity {
public:

  Place* System_OK;
  short* System_OK_Mobius_Mark;
  Place* Unsafe_state;
  short* Unsafe_state_Mobius_Mark;
  Place* Unsafe_state_2;
  short* Unsafe_state_2_Mobius_Mark;
  Place* Channel_OK;
  short* Channel_OK_Mobius_Mark;

  double* TheDistributionParameters;
  Repair_unsafeActivity();
  ~Repair_unsafeActivity();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // Repair_unsafeActivityActivity

  //List of user-specified place names
  Place* Channel_OK;
  Place* channel_failure;
  Place* Safe_state;
  Place* Unsafe_state;
  Place* Unsafe_state_2;
  Place* Configuration_state;
  Place* System_OK;
  Place* CCF_active;

  // Create instances of all actvities
  CCF_propagateActivity CCF_propagate;
  OK_failureActivity OK_failure;
  Safe_fail_OKActivity Safe_fail_OK;
  DiagnosisActivity_case1 Diagnosis_case1;
  DiagnosisActivity_case2 Diagnosis_case2;
  Repair_unsafeActivity Repair_unsafe;
  //Create instances of all groups 
  PreselectGroup ImmediateGroup;
  PostselectGroup CCF_propagateGroup;
  PostselectGroup DiagnosisGroup;
  SW_channelSAN();
  ~SW_channelSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end SW_channelSAN

#endif // SW_channelSAN_H_
